/*
 * file CqUser.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqUser
 *
 * (C) Copyright IBM Corporation 2006, 2014.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpPartialResultsException;
import com.ibm.rational.wvcm.stp.StpProperty;
import com.ibm.rational.wvcm.stp.StpResource;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * The proxy interface for information stored in a DbSet for a single user. A
 * users login, access privileges and some other information can be retrieved
 * and specified through this object. You can also subscribe and unsubscribe a
 * user to databases and upgrade user information in all the user databases that
 * the user is currently subscribed to.
 * <p>
 * Note that CqUser.doWriteProperties(...) writes dirty properties of this proxy
 * into the DbSet. For the new values to be visible to a user database,
 * {@link CqUserDb#doUpgradeUsersAndGroups(Feedback)} must be invoked on that
 * database or {@link CqUser#doUpgradeDatabases(Feedback)} must be invoked on
 * this user.
 * <p>
 * The user-friendly specification for the location of a user has the form
 * 
 * <pre>
 *  <b>cq.user:</b><i>&lt;user-name&gt;</i>@<i>&lt;db-set&gt;</i>
 * </pre>
 */
public interface CqUser extends CqResource, CqUserInfo
{
    /** The database set that contains this user resource */
    PropertyName<CqDbSet> DB_SET =
        new PropertyName<CqDbSet>(PROPERTY_NAMESPACE, "db-set");

    /**
     * Returns the value of the {@link #DB_SET DB_SET} property as defined by
     * this proxy.
     * 
     * @return A CqDbSet proxy for the database set that contains this resource.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DB_SET DB_SET} property.
     */
    CqDbSet getDbSet() throws WvcmException;

    /**
     * An enumeration of the authentication modes available to a user
     */
    enum AuthenticationMode implements StpExEnumeration
    {
        /** LDAP authentication is being used by this user */
        LDAP,

        /** Traditional ClearQuest authentication is being used by this user */
        CLEAR_QUEST
    }

    /**
     * Whether this user is authenticated via an LDAP login or via ClearQuest
     * login-name and password.
     */
    static PropertyName<AuthenticationMode> AUTHENTICATION_MODE =
        new PropertyName<AuthenticationMode>(PROPERTY_NAMESPACE,
                                             "authentication-mode");

    /**
     * Returns the value of the {@link #AUTHENTICATION_MODE AUTHENTICATION_MODE}
     * property as defined by this proxy.
     * 
     * @return An AuthenticationMode enumerator indicating what method is used
     *         to authenticate this user. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #AUTHENTICATION_MODE AUTHENTICATION_MODE} property.
     */
    AuthenticationMode getAuthenticationMode() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #AUTHENTICATION_MODE AUTHENTICATION_MODE} property of this proxy.
     * <p>
     * When setting the authentication mode to CLEAR_QUEST authentication, the
     * AUTHENTICATOR property specifies the new ClearQuest password.
     * <p>
     * When setting the authentication mode to LDAP authentication, the
     * AUTHENTICATOR property specifies the LDAP login name (a ClearQuest
     * password is not used for LDAP authentication).
     * 
     * @param mode An AuthenticationMode enumerator specifying the new
     *            authentication mode for this user.
     * 
     * @see #setLoginName(String)
     * @see #setAuthenticator(String)
     */
    void setAuthenticationMode(AuthenticationMode mode);

    /**
     * The user's e-mail address.
     */
    static PropertyName<String> EMAIL =
        new PropertyName<String>(PROPERTY_NAMESPACE, "email");

    /**
     * Returns the value of the {@link #EMAIL EMAIL} property as defined by this
     * proxy.
     * 
     * @return A String containing the email field of this user's profile. Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #EMAIL EMAIL} property.
     */
    String getEmail() throws WvcmException;

    /**
     * Defines a new value for the {@link #EMAIL EMAIL} property of this proxy.
     * 
     * @param email A String containing the new value for the email field of
     *            this user's profile
     */
    void setEmail(String email);

    /**
     * The full name of the user
     */
    static PropertyName<String> FULL_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "full-name");

    /**
     * Returns the value of the {@link #FULL_NAME FULL_NAME} property as defined
     * by this proxy.
     * 
     * @return A String containing the full name field of this user's profile.
     *         Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FULL_NAME FULL_NAME} property.
     */
    String getFullName() throws WvcmException;

    /**
     * Defines a new value for the {@link #FULL_NAME FULL_NAME} property of this
     * proxy.
     * 
     * @param fullName A String containing the new value for the full name field
     *            of this user's profile
     */
    void setFullName(String fullName) throws WvcmException;

    /**
     * A list of active user groups to which the user belongs.
     * 
     * The list can be empty.
     */
    static PropertyName<ResourceList<CqGroup>> GROUPS =
        new PropertyName<ResourceList<CqGroup>>(PROPERTY_NAMESPACE, "groups");

    /**
     * Returns the value of the {@link #GROUPS GROUPS} property as defined by
     * this proxy.
     * 
     * @return A ResourceList containing a CqGroup proxy for each group this
     *         user belongs to. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #GROUPS GROUPS} property.
     */
    ResourceList<CqGroup> getGroups() throws WvcmException;

    /**
     * The miscellaneous information field of the user's profile.
     */
    static PropertyName<String> MISCELLANEOUS_INFORMATION =
        new PropertyName<String>(PROPERTY_NAMESPACE,
                                 "miscellaneous-information");

    /**
     * Returns the value of the
     * {@link #MISCELLANEOUS_INFORMATION MISCELLANEOUS_INFORMATION} property as
     * defined by this proxy.
     * 
     * @return A String containing the content of the miscellaneous information
     *         field of this user's profile. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MISCELLANEOUS_INFORMATION MISCELLANEOUS_INFORMATION}
     *             property.
     */
    String getMiscellaneousInformation() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #MISCELLANEOUS_INFORMATION MISCELLANEOUS_INFORMATION} property of
     * this proxy.
     * 
     * @param miscInfo A String containing the new value for the miscellaneous
     *            information field of this user's profile
     */
    void setMiscellaneousInformation(String miscInfo);

    /**
     * The string used by ClearQuest to identify this user. Note: if LDAP
     * authentication is enabled for this user, the value of this property may
     * not actually be used for login, as the name of the property might
     * suggest.
     */
    static PropertyName<String> LOGIN_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "login-name");

    /**
     * Returns the value of the {@link #LOGIN_NAME LOGIN_NAME} property as
     * defined by this proxy.
     * 
     * @return A String containing the login name field of the user's profile.
     *         Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #LOGIN_NAME LOGIN_NAME} property.
     */
    String getLoginName() throws WvcmException;

    /**
     * Defines a new value for the {@link #LOGIN_NAME LOGIN_NAME} property of
     * this proxy. Note, changing the login name of a user changes the location
     * of the CqUser object for that user.
     * 
     * @param loginName A String containing the new value for the login-name
     *            field of this user's profile
     */
    void setLoginName(String loginName);

    /**
     * An encrypted version of the value placed in the AUTHENTICATOR property of
     * this CqUser resource.
     */
    static PropertyName<String> AUTHENTICATOR =
        new PropertyName<String>(PROPERTY_NAMESPACE, "authenticator");

    /**
     * Returns the value of the {@link #AUTHENTICATOR AUTHENTICATOR} property as
     * defined by this proxy.
     * 
     * @return A String containing an encrypted copy of the value of the
     *         AUTHENTICATOR property of this user resource. Will never be
     *         <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #AUTHENTICATOR AUTHENTICATOR} property.
     */
    String getAuthenticator() throws WvcmException;

    /**
     * Defines a new value for the {@link #AUTHENTICATOR AUTHENTICATOR} property
     * of this proxy. In LDAP AUTHENTICATION_MODE, or when changing to LDAP
     * AUTHENTICATION_MODE, this property specifies the LDAP login name to be
     * associated with this ClearQuest user. In CLEAR_QUEST AUTHENTICATION_MODE,
     * or when changing to CLEAR_QUEST AUTHENTICATION_MODE, this property
     * specifies the new ClearQuest password.
     * 
     * @param password A String containing the new value for the authenticator
     *            associated with this user.
     */
    void setAuthenticator(String password);

    /**
     * The phone number field from this user's profile.
     */
    static PropertyName<String> PHONE =
        new PropertyName<String>(PROPERTY_NAMESPACE, "phone");

    /**
     * Returns the value of the {@link #PHONE PHONE} property as defined by this
     * proxy.
     * 
     * @return A String containing the phone field of this user's profile. Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PHONE PHONE} property.
     */
    String getPhone() throws WvcmException;

    /**
     * Defines a new value for the {@link #PHONE PHONE} property of this proxy.
     * 
     * @param phone A String containing the new value for the phone field of
     *            this user's profile
     */
    void setPhone(String phone);

    /**
     * The databases to which the user is subscribed. If the user is subscribed
     * to all databases, this list will contain all databases.
     */
    static PropertyName<ResourceList<CqUserDb>> SUBSCRIBED_DATABASES =
        new PropertyName<ResourceList<CqUserDb>>(PROPERTY_NAMESPACE,
                                                 "subscribed-databases");

    /**
     * Returns the value of the
     * {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property as defined by
     * this proxy.
     * 
     * @return A ResourceList containing a CqUserDb resource for each user
     *         database to which this group is subscribed. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property.
     */
    ResourceList<CqUserDb> getSubscribedDatabases() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property of this
     * proxy.
     * 
     * @param value A ResourceList containing a CqUserDb proxy for each user
     *            database to which this user is now subscribed. May be <b>null</b>
     *            or empty to unsubscribe the user from all user databases.
     */
    void setSubscribedDatabases(ResourceList<CqUserDb> value);

    /**
     * Establishes the values to be added to and/or deleted from the
     * SUBSCRIBED_DATABASES property when that property is updated from this
     * proxy (via doWriteProperties or any other "do" method of this interface).
     * 
     * @param additions The list of CqUserDb proxies that must be in the
     *            SUBSCRIBED_DATABASES property after it has been updated from
     *            this proxy. Thus, it is OK for a value in this list to already
     *            be a member of the property value. This list may be empty or
     *            null if no values are to be added to the property. A value may
     *            not appear in both this list and the deletions list.
     * 
     * @param deletions The list of CqUserDb proxies that must not be in the
     *            SUBSCRIBED_DATABASES property after it has been updated from
     *            this proxy. Thus, it is OK for a value in this list to not be
     *            a current member of the property value. This list may be empty
     *            or null if no values are to be deleted from the property. A
     *            value may not appear in both this list and the additions list.
     */
    void setSubscribedDatabases(ResourceList<CqUserDb> additions,
                                ResourceList<CqUserDb> deletions);

    /**
     * Indication of whether or not this user's account is active
     */
    static PropertyName<Boolean> IS_ACTIVE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-active");

    /**
     * Returns the value of the {@link #IS_ACTIVE IS_ACTIVE} property as defined
     * by this proxy.
     * 
     * @return <b>true</b> if the user is active; <b>false</b> if not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA} property.
     */
    boolean getIsActive() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_ACTIVE IS_ACTIVE} property of this
     * proxy.
     * 
     * @param active <b>true</b> to make this user active; <b>false</b> to
     *            make the user inactive.
     */
    void setIsActive(boolean active);

    /**
     * The value of this property is an StpProperty.List containing an
     * StpProperty&lt;Boolean&gt; object for each user privilege defined by
     * ClearQuest. The Boolean value of each property indicates whether or not
     * this user has the privilege denoted by the property. The properties that
     * appear in this list are determined by the UserPrivilegeMaskType
     * enumeration defined by ClearQuest. The values in this enumeration known
     * at the time of release are defined by this API specification; others may
     * be subsequently defined and, if so, will also appear in this list.
     * <p>
     * Any of the privileges enumerated in the USER_PRIVILEGES property can be
     * granted or denied to the user by using Resource.setProperty to assign the
     * particular property either Boolean.TRUE or Boolean.FALSE.
     */
    static PropertyName<List<StpProperty<Boolean>>> USER_PRIVILEGES =
        new PropertyName<List<StpProperty<Boolean>>>(PROPERTY_NAMESPACE,
                                                     "user-privileges");

    /**
     * Returns the value of the {@link #USER_PRIVILEGES USER_PRIVILEGES}
     * property as defined by this proxy.
     * 
     * @return A List containing one StpProperty object for each privilege
     *         defined by ClearQuest. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #USER_PRIVILEGES USER_PRIVILEGES} property.
     */
    List<StpProperty<Boolean>> getUserPrivileges() throws WvcmException;

    /**
     * Answers whether or not this user can create and manage dynamic lists.
     */
    static PropertyName<Boolean> IS_DYNAMIC_LIST_ADMIN =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-dynamic-list-admin");

    /**
     * Returns the value of the
     * {@link #IS_DYNAMIC_LIST_ADMIN IS_DYNAMIC_LIST_ADMIN} property as defined
     * by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit dynamic choice
     *         lists; <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_DYNAMIC_LIST_ADMIN IS_DYNAMIC_LIST_ADMIN}
     *             property.
     */
    boolean getIsDynamicListAdmin() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_DYNAMIC_LIST_ADMIN IS_DYNAMIC_LIST_ADMIN} property of this
     * proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit dynamic choice
     *            lists; <b>false</b> to deny the user this ability.
     */
    void setIsDynamicListAdmin(boolean permitted);

    /**
     * Answers whether or not this user can create, delete, or update resources
     * in the public folders heirarcy used for queries, reports, and charts.
     */
    static PropertyName<Boolean> IS_PUBLIC_FOLDER_ADMIN =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-public-folder-admin");

    /**
     * Returns the value of the
     * {@link #IS_PUBLIC_FOLDER_ADMIN IS_PUBLIC_FOLDER_ADMIN} property as
     * defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit public folders;
     *         <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_PUBLIC_FOLDER_ADMIN IS_PUBLIC_FOLDER_ADMIN}
     *             property.
     */
    boolean getIsPublicFolderAdmin() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_PUBLIC_FOLDER_ADMIN IS_PUBLIC_FOLDER_ADMIN} property of this
     * proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit public folders;
     *            <b>false</b> to deny the user this ability.
     */
    void setIsPublicFolderAdmin(boolean permitted);

    /**
     * Answers whether or not this user can access and manage secure records and
     * fields as well as edit the context group list field for a security context
     * record and view all records.
     */
    static PropertyName<Boolean> IS_SECURITY_ADMIN =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-security-admin");

    /**
     * Returns the value of the {@link #IS_SECURITY_ADMIN IS_SECURITY_ADMIN}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit security
     *         components; <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SECURITY_ADMIN IS_SECURITY_ADMIN } property.
     */
    boolean getIsSecurityAdmin() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_SECURITY_ADMIN IS_SECURITY_ADMIN }
     * property of this proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit security
     *            components; <b>false</b> to deny the user this ability.
     */
    void setIsSecurityAdmin(boolean permitted);

    /**
     * Answers whether or not this user can create (either from scratch or
     * through modification of an existing query) a query defined by a raw SQL
     * string.
     */
    static PropertyName<Boolean> IS_RAW_SQL_WRITER =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-raw-sql-writer");

    /**
     * Returns the value of the {@link #IS_RAW_SQL_WRITER IS_RAW_SQL_WRITER }
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit raw SQL in queries;
     *         <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_RAW_SQL_WRITER IS_RAW_SQL_WRITER } property.
     */
    boolean getIsRawSqlWriter() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_RAW_SQL_WRITER IS_RAW_SQL_WRITER }
     * property of this proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit raw SQL in
     *            queries; <b>false</b> to deny the user this ability.
     */
    void setIsRawSqlWriter(boolean permitted);

    /**
     * Answers whether or not this user can view information about all users and
     * groups in user databases.
     */
    static PropertyName<Boolean> IS_ALL_USERS_VISIBLE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-all-users-visible");

    /**
     * Returns the value of the
     * {@link #IS_ALL_USERS_VISIBLE IS_ALL_USERS_VISIBLE } property as defined
     * by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to see all users and groups; <b>false</b>
     *         if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_ALL_USERS_VISIBLE IS_ALL_USERS_VISIBLE } property.
     */
    boolean getIsAllUsersVisible() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_ALL_USERS_VISIBLE IS_ALL_USERS_VISIBLE } property of this
     * proxy.
     * 
     * @param permitted <b>true</b> to permit this user to see all users;
     *            <b>false</b> to deny the user this ability.
     */
    void setIsAllUsersVisible(boolean permitted);

    /**
     * Answers whether or not this user has multisite administrator privileges.
     */
    static PropertyName<Boolean> IS_MULTI_SITE_ADMIN =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-multi-site-admin");

    /**
     * Returns the value of the
     * {@link #IS_MULTI_SITE_ADMIN IS_MULTI_SITE_ADMIN } property as defined by
     * this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit multisite
     *         components; <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_MULTI_SITE_ADMIN IS_MULTI_SITE_ADMIN } property.
     */
    boolean getIsMultiSiteAdmin() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_MULTI_SITE_ADMIN IS_MULTI_SITE_ADMIN } property of this proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit multi-site
     *            components; <b>false</b> to deny the user this ability.
     */
    void setIsMultiSiteAdmin(boolean permitted);

    /**
     * Answers whether or not this user has been granted all privileges and can
     * also create and delete databases and schemas and edit ClearQuest Web
     * settings. The administrator account has this privilege.
     */
    static PropertyName<Boolean> IS_SUPER_USER =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-super-user");

    /**
     * Returns the value of the {@link #IS_SUPER_USER IS_SUPER_USER } property
     * as defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to be super user; <b>false</b>
     *         if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SUPER_USER IS_SUPER_USER } property.
     */
    boolean getIsSuperUser() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_SUPER_USER } property of this
     * proxy.
     * 
     * @param permitted <b>true</b> to permit this user to be super user;
     *            <b>false</b> to deny the user this ability.
     */
    void setIsSuperUser(boolean permitted);

    /**
     * Answers whether or not this user can create and modify schemas, update
     * existing databases, and create, modify, and save public queries, charts, and
     * reports (but not perform user administrator tasks).
     */
    PropertyName<Boolean> IS_APP_BUILDER =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-app-builder");

    /**
     * Returns the value of the {@link #IS_APP_BUILDER IS_APP_BUILDER } property
     * as defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit schemas; <b>false</b>
     *         if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_APP_BUILDER IS_APP_BUILDER } property.
     */
    boolean getIsAppBuilder() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_APP_BUILDER IS_APP_BUILDER }
     * property of this proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit schemas;
     *            <b>false</b> to deny the user this ability.
     */
    void setIsAppBuilder(boolean permitted);

    /**
     * Answers whether or not this user can create users and user groups and
     * assign and modify their user-access privileges.
     */
    PropertyName<Boolean> IS_USER_MAINTAINER =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-user-maintainer");

    /**
     * Returns the value of the {@link #IS_USER_MAINTAINER IS_USER_MAINTAINER }
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the user is permitted to edit user and groups
     *         artifacts; <b>false</b> if the user may not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_USER_MAINTAINER IS_USER_MAINTAINER } property.
     */
    boolean getIsUserMaintainer() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_USER_MAINTAINER IS_USER_MAINTAINER } property of this proxy.
     * 
     * @param permitted <b>true</b> to permit this user to edit user and group
     *            artifacts; <b>false</b> to deny the user this ability.
     */
    void setIsUserMaintainer(boolean permitted);

    /**
     * Answers whether or not this user is automatically subscribed to all
     * databases of the DbSet.
     */
    static PropertyName<Boolean> IS_SUBSCRIBED_TO_ALL_DATABASES =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-subscribed-to-all-databases");

    /**
     * Returns the value of the
     * {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES }
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the user is subscribed to all databases; <b>false</b>
     *         if the user is subscribed only to those databases enumerated on
     *         the SUBSCRIBED_DATABASES list.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES }
     *             property.
     */
    boolean getIsSubscribedToAllDatabases() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES }
     * property of this proxy.
     * 
     * @param permitted <b>true</b> to automatically subscribe the user to all
     *            databases current and future; <b>false</b> to require the
     *            user to subscribe to each database individually.
     */
    void setIsSubscribedToAllDatabases(boolean permitted);

    /**
     * The replica that masters this user
     */
    PropertyName<CqReplica> CQ_MASTER_REPLICA =
        new PropertyName<CqReplica>(PROPERTY_NAMESPACE, "cq-master-replica");

    /**
     * Returns the value of the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA} property
     * as defined by this proxy.
     * 
     * @return A CqReplica proxy for the replica that masters this user. Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA} property.
     */
    CqReplica getCqMasterReplica() throws WvcmException;

    /**
     * Defines a new value for the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA}
     * property of this proxy.
     * 
     * @param value A CqReplica proxy specifying the replica that is to assume
     *            mastership of this user resource.
     */
    void setCqMasterReplica(CqReplica value);

    /**
     * Creates a new user whose ClearQuest login name will be the same as the
     * Name field of this proxy's location. The new user is created in the DbSet
     * named by the Repo field of that location. The location is of the form
     * 
     * <pre>
     * cq.user:&lt;login-name>@&lt;db-set-name>
     * </pre>
     * 
     * If the LOGIN_NAME property is not set, it will default to the Name field
     * of this proxy's location.
     * <p>
     * If the AUTHENTICATION_MODE is CLEAR_QUEST, the value of the LOGIN_NAME
     * property of this proxy, if set, is ignored and forced to be the same as
     * the Name field of this proxy's location. The AUTHENTICATOR should be the
     * user's ClearQuest password.
     * <p>
     * If the AUTHENTICATION_MODE is LDAP, then the AUTHENTICATOR should be set
     * to the LDAP authentication name that is to correspond to this user. An
     * LDAP attribute value is copied from the LDAP user account to a ClearQuest
     * user profile field as specified by the value of
     * {@link CqDb#LDAP_PROPERTY}. This method first checks the DbSet to ensure
     * that there is no conflict with another active LDAP-enabled user's
     * LDAP_PROPERTY value to ensure that the values are unique across active
     * LDAP enabled users. If the LDAP_PROPERTY is LOGIN_NAME_FIELD, the
     * login-name must be identical to LOGIN_NAME.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this new resource, whose properties are specified by
     *         feedback.
     * @throws WvcmException ReasonCode:
     *             <li>{@link javax.wvcm.WvcmException.ReasonCode#RESOURCE_ALREADY_EXISTS_AT_LOCATION RESOURCE_ALREADY_EXISTS_AT_LOCATION}:
     *             If a resource already exists at the location of this
     *             Resource, the request MUST fail.
     *             <li>{@link javax.wvcm.WvcmException.ReasonCode#CANNOT_CREATE_AT_THIS_LOCATION CANNOT_CREATE_AT_THIS_LOCATION}:
     *             If the location of this Group does not identify a valid
     *             location in which to create a group, the request MUST fail.
     *             Groups must be created in a DbSet
     */
    CqUser doCreateUser(Feedback feedback) throws WvcmException;

    /**
     * Attempts to upgrade the user account information in all the user
     * databases that this user is currently subscribed to.
     * 
     * Any dirty properties of this proxy are written to the DbSet before the
     * upgrading of user databases is attempted. If the property updates fail,
     * no attempt is made to upgrade the subscribed databases.
     * 
     * @param feedback Specifies optional feedback to the caller. The databases
     *            being upgraded are considered to be resources modified by this
     *            operation and will generate calls to
     *            {@link javax.wvcm.DetailedFeedback#notifyIsModified(javax.wvcm.Resource)}
     *            if requested.
     * @return A proxy for this resource after the upgrade is complete, with
     *         properties as specified by feedback.
     * @throws WvcmException if any dirty properties could not be updated.
     * @throws StpPartialResultsException if any of the subscribed databases
     *             could not be upgraded, with the failed updates indicated by
     *             nested exceptions of this exception.
     */
    CqUser doUpgradeDatabases(Feedback feedback) throws WvcmException;
}
